@@ -202,7 +202,7 @@ smtp_auth_check_greeting (GDataInputStream *input, GCancellable *cancellable, GE
 /* ---------------------------------------------------------------------------------------------------- */
 
 static char *
-imap_auth_escape0 (const char *str,
+smtp_auth_escape0 (const char *str,
                    size_t     *length)
 {
   GString *ret;
@@ -369,15 +369,12 @@ goa_smtp_auth_class_init (GoaSmtpAuthClass *klass)
 /**
  * goa_smtp_auth_new:
  * @domain: The domain to use.
- * @username: (nullable): The user name to use.
- * @password: (nullable): The password to use.
+ * @username: The user name to use.
+ * @password: The password to use.
  *
  * Creates a new #GoaMailAuth to be used for username/password
  * authentication using LOGIN or PLAIN over SMTP.
  *
- * If either @username or @password are %NULL, authentication will only
- * succeed if the service supports it.
- *
  * Returns: (type GoaSmtpAuth): A #GoaSmtpAuth. Free with
  * g_object_unref().
  */
@@ -387,8 +384,8 @@ goa_smtp_auth_new (const gchar       *domain,
                    const gchar       *password)
 {
   g_return_val_if_fail (domain != NULL && domain[0] != '\0', NULL);
-  g_return_val_if_fail (username != NULL || (username == NULL && password == NULL), NULL);
-  g_return_val_if_fail (password == NULL || password[0] != '\0', NULL);
+  g_return_val_if_fail (username != NULL, NULL);
+  g_return_val_if_fail (password != NULL && password[0] != '\0', NULL);
 
   return GOA_MAIL_AUTH (g_object_new (GOA_TYPE_SMTP_AUTH,
                                       "domain", domain,
@@ -487,14 +484,6 @@ goa_smtp_auth_run_sync (GoaMailAuth         *auth,
       ret = TRUE;
       goto out;
     }
-  else if (self->username == NULL && self->password == NULL)
-    {
-      g_set_error (error,
-                   GOA_ERROR,
-                   GOA_ERROR_NOT_AUTHORIZED,
-                   _("Authentication failed"));
-      goto out;
-    }
   else if (!self->login_supported && !self->plain_supported)
     {
       g_set_error (error,
@@ -514,8 +503,8 @@ goa_smtp_auth_run_sync (GoaMailAuth         *auth,
 
       /* AUTH PLAIN */
 
-      username_esc = imap_auth_escape0 (self->username, &username_len);
-      password_esc = imap_auth_escape0 (self->password, &password_len);
+      username_esc = smtp_auth_escape0 (self->username, &username_len);
+      password_esc = smtp_auth_escape0 (self->password, &password_len);
       auth_arg_plain = g_strdup_printf ("%s%c%s%c%s", username_esc, '\0', username_esc, '\0', password_esc);
       auth_arg_plain_len = 2 * username_len + 2 + password_len;
       auth_arg_base64 = g_base64_encode ((guchar *) auth_arg_plain, auth_arg_plain_len);
@@ -530,7 +519,7 @@ goa_smtp_auth_run_sync (GoaMailAuth         *auth,
     {
       /* AUTH LOGIN */
 
-      auth_arg_plain = imap_auth_escape0 (self->username, &auth_arg_plain_len);
+      auth_arg_plain = smtp_auth_escape0 (self->username, &auth_arg_plain_len);
       auth_arg_base64 = g_base64_encode ((guchar *) auth_arg_plain, auth_arg_plain_len);
 
       request = g_strdup_printf ("AUTH LOGIN %s\r\n", auth_arg_base64);
@@ -549,7 +538,7 @@ goa_smtp_auth_run_sync (GoaMailAuth         *auth,
       g_free (auth_arg_plain);
       g_free (auth_arg_base64);
 
-      auth_arg_plain = imap_auth_escape0 (self->password, &auth_arg_plain_len);
+      auth_arg_plain = smtp_auth_escape0 (self->password, &auth_arg_plain_len);
       auth_arg_base64 = g_base64_encode ((guchar *) auth_arg_plain, auth_arg_plain_len);
 
       request = g_strdup_printf ("%s\r\n", auth_arg_base64);
